@@ -526,6 +526,28 @@ namespace TagLib.Ape {
 					items.RemoveAt (i);
 		}
 		
+		/// <summary>
+		/// Checks if an item exists.
+		/// </summary>
+		/// <param name="key">
+		///    A <see cref="string" /> object containing the key of the
+		///    item to check.
+		/// </param>
+		/// <returns>
+		///    Returns <see langword="true"/> if the <paramref name="key"/>
+		///    exists - else <see langword="false"/> is returned.
+		/// </returns>
+		/// <exception cref="ArgumentNullException">
+		///    <paramref name="key" /> is <see langword="null" />.
+		/// </exception>
+		public bool HasItem(string key)
+		{
+			if (key == null)
+				throw new ArgumentNullException("key");
+			
+			return GetItemIndex(key) >= 0;
+		}
+		
 		/// <summary>
 		///    Renders the current instance as a raw APEv2 tag.
 		/// </summary>
@@ -572,7 +594,7 @@ namespace TagLib.Ape {
 		/// <exception cref="ArgumentNullException">
 		///    <paramref name="file" /> is <see langword="null" />.
 		/// </exception>
-		/// <exception name="ArgumentOutOfRangeException">
+		/// <exception cref="ArgumentOutOfRangeException">
 		///    <paramref name="position" /> is less than 0 or greater
 		///    than the size of the file.
 		/// </exception>
@@ -800,16 +822,30 @@ namespace TagLib.Ape {
 		///    langword="null" /> if no value is present.
 		/// </value>
 		/// <remarks>
-		///    This property is implemented using the "TITLE" item.
+		///    This property is implemented using the "Title" item.
 		/// </remarks>
 		public override string Title {
-			get {
-				Item item = GetItem ("TITLE");
-				return item != null ? item.ToString () : null;
-			}
-			set {SetValue ("TITLE", value);}
+			get { return GetItemAsString ("Title"); }
+			set {SetValue ("Title", value);}
 		}
-		
+
+		/// <summary>
+		///    Gets and sets the sort names of the Title of the
+		///    media represented by the current instance.
+		/// </summary>
+		/// <value>
+		///    A <see cref="string" /> containing the sort names for
+		///    the Title of the media described by the current instance,
+		///    or null if no value is present. 
+		/// </value>
+		/// <remarks>
+		///    This property is implemented using the "TitleSort" item.
+		/// </remarks>
+		public override string TitleSort {
+			get { return GetItemAsString ("TitleSort"); }
+			set { SetValue ("TitleSort", value); }
+		}
+
 		/// <summary>
 		///    Gets and sets the performers or artists who performed in
 		///    the media described by the current instance.
@@ -821,13 +857,32 @@ namespace TagLib.Ape {
 		///    present.
 		/// </value>
 		/// <remarks>
-		///    This property is implemented using the "ARTIST" item.
+		///    This property is implemented using the "Artist" item.
 		/// </remarks>
 		public override string [] Performers {
-			get {return GetItemAsStrings ("ARTIST");}
-			set {SetValue ("ARTIST", value);}
+			get {return GetItemAsStrings ("Artist");}
+			set {SetValue ("Artist", value);}
 		}
-		
+
+		/// <summary>
+		///    Gets and sets the sort names of the performers or artists
+		///    who performed in the media described by the current instance.
+		/// </summary>
+		/// <value>
+		///    A <see cref="string" /> array containing the sort names for
+		///    the performers or artists who performed in the media
+		///    described by the current instance, or an empty array if
+		///    no value is present. 
+		/// </value>
+		/// <remarks>
+		///    This property is implemented using the "ArtistSort" field.
+		///    http://musicbrainz.org/doc/PicardTagMapping
+		/// </remarks>
+		public override string[] PerformersSort {
+			get { return GetItemAsStrings ("ArtistSort"); }
+			set { SetValue ("ArtistSort", value); }
+		}
+
 		/// <summary>
 		///    Gets and sets the band or artist who is credited in the
 		///    creation of the entire album or collection containing the
@@ -840,14 +895,47 @@ namespace TagLib.Ape {
 		///    instance or an empty array if no value is present.
 		/// </value>
 		/// <remarks>
-		///    This property is implemented using the "ALBUM ARTIST"
-		///    item.
+		///    This property is implemented using the "Album Artist"
+		///    item, and "AlbumArtist" as a backup property if it exists.
 		/// </remarks>
 		public override string [] AlbumArtists {
-			get {return GetItemAsStrings ("ALBUM ARTIST");}
-			set {SetValue ("ALBUM ARTIST", value);}
+			get {
+				string[] list = GetItemAsStrings("Album Artist");
+				if (list.Length == 0)
+					list = GetItemAsStrings("AlbumArtist");
+				return list;
+			}
+			set {
+				SetValue("Album Artist", value);
+				// compatibility
+				if (HasItem("AlbumArtist"))
+					SetValue("AlbumArtist", value);
+				}
 		}
-		
+
+		/// <summary>
+		///    Gets and sets the sort names for the band or artist who
+		///    is credited in the creation of the entire album or
+		///    collection containing the media described by the
+		///    current instance.
+		/// </summary>
+		/// <value>
+		///    A <see cref="string" /> array containing the sort names
+		///    for the band or artist who is credited in the creation
+		///    of the entire album or collection containing the media
+		///    described by the current instance or an empty array if
+		///    no value is present.
+		/// </value>
+		/// <remarks>
+		///    This property is implemented using the "AlbumArtistSort"
+		///    field.
+		///    http://musicbrainz.org/doc/PicardTagMapping
+		/// </remarks>
+		public override string[] AlbumArtistsSort {
+			get { return GetItemAsStrings ("AlbumArtistSort"); }
+			set { SetValue ("AlbumArtistSort", value); }
+		}
+
 		/// <summary>
 		///    Gets and sets the composers of the media represented by
 		///    the current instance.
@@ -858,13 +946,32 @@ namespace TagLib.Ape {
 		///    array if no value is present.
 		/// </value>
 		/// <remarks>
-		///    This property is implemented using the "COMPOSER" item.
+		///    This property is implemented using the "Composer" item.
 		/// </remarks>
 		public override string [] Composers {
-			get {return GetItemAsStrings ("COMPOSER");}
-			set {SetValue ("COMPOSER", value);}
+			get {return GetItemAsStrings ("Composer");}
+			set {SetValue ("Composer", value);}
 		}
-		
+
+		/// <summary>
+		///    Gets and sets the sort names for the composers of
+		///    the media described by the current instance.
+		/// </summary>
+		/// <value>
+		///    A <see cref="string" /> array containing the sort names
+		///    for the composer of the media described by the current
+		///    instance or an empty array if no value is present.
+		/// </value>
+		/// <remarks>
+		///    This property is implemented using the "ComposerSort"
+		///    field.
+		///    http://musicbrainz.org/doc/PicardTagMapping
+		/// </remarks>
+		public override string[] ComposersSort {
+			get { return GetItemAsStrings ("ComposerSort"); }
+			set { SetValue ("ComposerSort", value); }
+		}
+
 		/// <summary>
 		///    Gets and sets the album of the media represented by the
 		///    current instance.
@@ -875,13 +982,32 @@ namespace TagLib.Ape {
 		///    langword="null" /> if no value is present.
 		/// </value>
 		/// <remarks>
-		///    This property is implemented using the "ALBUM" item.
+		///    This property is implemented using the "Album" item.
 		/// </remarks>
 		public override string Album {
-			get {return GetItemAsString ("ALBUM");}
-			set {SetValue ("ALBUM", value);}
+			get {return GetItemAsString ("Album");}
+			set {SetValue ("Album", value);}
 		}
-		
+
+		/// <summary>
+		///    Gets and sets the sort names for the Album Title of
+		///    the media described by the current instance.
+		/// </summary>
+		/// <value>
+		///    A <see cref="string" /> containing the sort name of 
+		///    the Album Title of the media described by the current
+		///    instance or null if no value is present.
+		/// </value>
+		/// <remarks>
+		///    This property is implemented using the "AlbumSort"
+		///    field.
+		///    http://musicbrainz.org/doc/PicardTagMapping
+		/// </remarks>
+		public override string AlbumSort {
+			get { return GetItemAsString ("AlbumSort"); }
+			set { SetValue ("AlbumSort", value); }
+		}
+
 		/// <summary>
 		///    Gets and sets a user comment on the media represented by
 		///    the current instance.
@@ -892,11 +1018,11 @@ namespace TagLib.Ape {
 		///    langword="null" /> if no value is present.
 		/// </value>
 		/// <remarks>
-		///    This property is implemented using the "COMMENT" item.
+		///    This property is implemented using the "Comment" item.
 		/// </remarks>
 		public override string Comment {
-			get {return GetItemAsString ("COMMENT");}
-			set {SetValue ("COMMENT", value);}
+			get {return GetItemAsString ("Comment");}
+			set {SetValue ("Comment", value);}
 		}
 		
 		/// <summary>
@@ -909,11 +1035,11 @@ namespace TagLib.Ape {
 		///    array if no value is present.
 		/// </value>
 		/// <remarks>
-		///    This property is implemented using the "GENRE" item.
+		///    This property is implemented using the "Genre" item.
 		/// </remarks>
 		public override string [] Genres {
-			get {return GetItemAsStrings ("GENRE");}
-			set {SetValue ("GENRE", value);}
+			get {return GetItemAsStrings ("Genre");}
+			set {SetValue ("Genre", value);}
 		}
 		
 		/// <summary>
@@ -926,11 +1052,11 @@ namespace TagLib.Ape {
 		///    if no value is present.
 		/// </value>
 		/// <remarks>
-		///    This property is implemented using the "YEAR" item.
+		///    This property is implemented using the "Year" item.
 		/// </remarks>
 		public override uint Year {
 			get {
-				string text = GetItemAsString ("YEAR");
+				string text = GetItemAsString ("Year");
 				
 				if (text == null || text.Length == 0)
 					return 0;
@@ -944,7 +1070,7 @@ namespace TagLib.Ape {
 				
 				return 0;
 			}
-			set {SetValue ("YEAR", value, 0);}
+			set {SetValue ("Year", value, 0);}
 		}
 		
 		/// <summary>
@@ -957,11 +1083,11 @@ namespace TagLib.Ape {
 		///    containing album or zero if not specified.
 		/// </value>
 		/// <remarks>
-		///    This property is implemented using the "TRACK" item.
+		///    This property is implemented using the "Track" item.
 		/// </remarks>
 		public override uint Track {
-			get {return GetItemAsUInt32 ("TRACK", 0);}
-			set {SetValue ("TRACK", value, TrackCount);}
+			get {return GetItemAsUInt32 ("Track", 0);}
+			set {SetValue ("Track", value, TrackCount);}
 		}
 		
 		/// <summary>
@@ -974,11 +1100,11 @@ namespace TagLib.Ape {
 		///    instance or zero if not specified.
 		/// </value>
 		/// <remarks>
-		///    This property is implemented using the "TRACK" item.
+		///    This property is implemented using the "Track" item.
 		/// </remarks>
 		public override uint TrackCount {
-			get {return GetItemAsUInt32 ("TRACK", 1);}
-			set {SetValue ("TRACK", Track, value);}
+			get {return GetItemAsUInt32 ("Track", 1);}
+			set {SetValue ("Track", Track, value);}
 		}
 		
 		/// <summary>
@@ -991,11 +1117,11 @@ namespace TagLib.Ape {
 		///    in the boxed set.
 		/// </value>
 		/// <remarks>
-		///    This property is implemented using the "DISC" item.
+		///    This property is implemented using the "Disc" item.
 		/// </remarks>
 		public override uint Disc {
-			get {return GetItemAsUInt32 ("DISC", 0);}
-			set {SetValue ("DISC", value, DiscCount);}
+			get {return GetItemAsUInt32 ("Disc", 0);}
+			set {SetValue ("Disc", value, DiscCount);}
 		}
 		
 		/// <summary>
@@ -1008,11 +1134,11 @@ namespace TagLib.Ape {
 		///    current instance or zero if not specified.
 		/// </value>
 		/// <remarks>
-		///    This property is implemented using the "DISC" item.
+		///    This property is implemented using the "Disc" item.
 		/// </remarks>
 		public override uint DiscCount {
-			get {return GetItemAsUInt32 ("DISC", 1);}
-			set {SetValue ("DISC", Disc, value);}
+			get {return GetItemAsUInt32 ("Disc", 1);}
+			set {SetValue ("Disc", Disc, value);}
 		}
 		
 		/// <summary>
@@ -1025,11 +1151,11 @@ namespace TagLib.Ape {
 		///    or <see langword="null" /> if no value is present.
 		/// </value>
 		/// <remarks>
-		///    This property is implemented using the "LYRICS" item.
+		///    This property is implemented using the "Lyrics" item.
 		/// </remarks>
 		public override string Lyrics {
-			get {return GetItemAsString ("LYRICS");}
-			set {SetValue ("LYRICS", value);}
+			get {return GetItemAsString ("Lyrics");}
+			set {SetValue ("Lyrics", value);}
 		}
 		
 		/// <summary>
@@ -1042,11 +1168,11 @@ namespace TagLib.Ape {
 		///    to or <see langword="null" /> if no value is present.
 		/// </value>
 		/// <remarks>
-		///    This property is implemented using the "GROUPING" item.
+		///    This property is implemented using the "Grouping" item.
 		/// </remarks>
 		public override string Grouping {
-			get {return GetItemAsString ("GROUPING");}
-			set {SetValue ("GROUPING", value);}
+			get {return GetItemAsString ("Grouping");}
+			set {SetValue ("Grouping", value);}
 		}
 		
 		/// <summary>
@@ -1059,11 +1185,11 @@ namespace TagLib.Ape {
 		///    current instance, or zero if not specified.
 		/// </value>
 		/// <remarks>
-		///    This property is implemented using the "TEMPO" item.
+		///    This property is implemented using the "BPM" item.
 		/// </remarks>
 		public override uint BeatsPerMinute {
 			get {
-				string text = GetItemAsString ("TEMPO");
+				string text = GetItemAsString ("BPM");
 				
 				if (text == null)
 					return 0;
@@ -1075,7 +1201,7 @@ namespace TagLib.Ape {
 				
 				return 0;
 			}
-			set {SetValue ("TEMPO", value, 0);}
+			set {SetValue ("BPM", value, 0);}
 		}
 		
 		/// <summary>
@@ -1088,11 +1214,11 @@ namespace TagLib.Ape {
 		///    instance or <see langword="null" /> if no value present.
 		/// </value>
 		/// <remarks>
-		///    This property is implemented using the "CONDUCTOR" item.
+		///    This property is implemented using the "Conductor" item.
 		/// </remarks>
 		public override string Conductor {
-			get {return GetItemAsString ("CONDUCTOR");}
-			set {SetValue ("CONDUCTOR", value);}
+			get {return GetItemAsString ("Conductor");}
+			set {SetValue ("Conductor", value);}
 		}
 		
 		/// <summary>
@@ -1105,13 +1231,193 @@ namespace TagLib.Ape {
 		///    instance or <see langword="null" /> if no value present.
 		/// </value>
 		/// <remarks>
-		///    This property is implemented using the "COPYRIGHT" item.
+		///    This property is implemented using the "Copyright" item.
 		/// </remarks>
 		public override string Copyright {
-			get {return GetItemAsString ("COPYRIGHT");}
-			set {SetValue ("COPYRIGHT", value);}
+			get {return GetItemAsString ("Copyright");}
+			set {SetValue ("Copyright", value);}
 		}
-		
+
+		/// <summary>
+		///    Gets and sets the MusicBrainz Artist ID of the media
+		///    represented by the current instance.
+		/// </summary>
+		/// <value>
+		///    A <see cref="string" /> object containing the MusicBrainz
+		///    ArtistID for the media represented by the current instance
+		///    or <see langword="null" /> if no value is present.
+		/// </value>
+		/// <remarks>
+		///    This property is implemented using the "MUSICBRAINZ_ARTISTID" item.
+		///    http://musicbrainz.org/doc/PicardTagMapping
+		/// </remarks>
+		public override string MusicBrainzArtistId {
+			get {return GetItemAsString ("MUSICBRAINZ_ARTISTID");}
+			set {SetValue ("MUSICBRAINZ_ARTISTID", value);}
+		}
+
+		/// <summary>
+		///    Gets and sets the MusicBrainz Release ID of the media
+		///    represented by the current instance.
+		/// </summary>
+		/// <value>
+		///    A <see cref="string" /> object containing the MusicBrainz
+		///    ReleaseID for the media represented by the current instance
+		///    or <see langword="null" /> if no value is present.
+		/// </value>
+		/// <remarks>
+		///    This property is implemented using the "MUSICBRAINZ_ALBUMID" item.
+		///    http://musicbrainz.org/doc/PicardTagMapping
+		/// </remarks>
+		public override string MusicBrainzReleaseId {
+			get {return GetItemAsString ("MUSICBRAINZ_ALBUMID");}
+			set {SetValue ("MUSICBRAINZ_ALBUMID", value);}
+		}
+
+		/// <summary>
+		///    Gets and sets the MusicBrainz Release Artist ID of the media
+		///    represented by the current instance.
+		/// </summary>
+		/// <value>
+		///    A <see cref="string" /> object containing the MusicBrainz
+		///    ReleaseArtistID for the media represented by the current instance
+		///    or <see langword="null" /> if no value is present.
+		/// </value>
+		/// <remarks>
+		///    This property is implemented using the "MUSICBRAINZ_ALBUMARTISTID" item.
+		///    http://musicbrainz.org/doc/PicardTagMapping
+		/// </remarks>
+		public override string MusicBrainzReleaseArtistId {
+			get {return GetItemAsString ("MUSICBRAINZ_ALBUMARTISTID");}
+			set {SetValue ("MUSICBRAINZ_ALBUMARTISTID", value);}
+		}
+
+		/// <summary>
+		///    Gets and sets the MusicBrainz Track ID of the media
+		///    represented by the current instance.
+		/// </summary>
+		/// <value>
+		///    A <see cref="string" /> object containing the MusicBrainz
+		///    TrackID for the media represented by the current instance
+		///    or <see langword="null" /> if no value is present.
+		/// </value>
+		/// <remarks>
+		///    This property is implemented using the "MUSICBRAINZ_TRACKID" item.
+		///    http://musicbrainz.org/doc/PicardTagMapping
+		/// </remarks>
+		public override string MusicBrainzTrackId {
+			get {return GetItemAsString ("MUSICBRAINZ_TRACKID");}
+			set {SetValue ("MUSICBRAINZ_TRACKID", value);}
+		}
+
+		/// <summary>
+		///    Gets and sets the MusicBrainz Disc ID of the media
+		///    represented by the current instance.
+		/// </summary>
+		/// <value>
+		///    A <see cref="string" /> object containing the MusicBrainz
+		///    DiscID for the media represented by the current instance
+		///    or <see langword="null" /> if no value is present.
+		/// </value>
+		/// <remarks>
+		///    This property is implemented using the "MUSICBRAINZ_DISCID" item.
+		///    http://musicbrainz.org/doc/PicardTagMapping
+		/// </remarks>
+		public override string MusicBrainzDiscId {
+			get {return GetItemAsString ("MUSICBRAINZ_DISCID");}
+			set {SetValue ("MUSICBRAINZ_DISCID", value);}
+		}
+
+		/// <summary>
+		///    Gets and sets the MusicIP PUID of the media
+		///    represented by the current instance.
+		/// </summary>
+		/// <value>
+		///    A <see cref="string" /> object containing the MusicIPPUID
+		///    for the media represented by the current instance
+		///    or <see langword="null" /> if no value is present.
+		/// </value>
+		/// <remarks>
+		///    This property is implemented using the "MUSICIP_PUID" item.
+		///    http://musicbrainz.org/doc/PicardTagMapping
+		/// </remarks>
+		public override string MusicIpId {
+			get {return GetItemAsString ("MUSICIP_PUID");}
+			set {SetValue ("MUSICIP_PUID", value);}
+		}
+
+		/// <summary>
+		///    Gets and sets the Amazon ID of the media
+		///    represented by the current instance.
+		/// </summary>
+		/// <value>
+		///    A <see cref="string" /> object containing the AmazonID
+		///    for the media represented by the current instance
+		///    or <see langword="null" /> if no value is present.
+		/// </value>
+		/// <remarks>
+		///    This property is implemented using the "ASIN" item.
+		///    http://musicbrainz.org/doc/PicardTagMapping
+		/// </remarks>
+		public override string AmazonId {
+			get {return GetItemAsString ("ASIN");}
+			set {SetValue ("ASIN", value);}
+		}
+
+		/// <summary>
+		///    Gets and sets the MusicBrainz Release Status of the media
+		///    represented by the current instance.
+		/// </summary>
+		/// <value>
+		///    A <see cref="string" /> object containing the MusicBrainz
+		///    ReleaseStatus for the media represented by the current instance
+		///    or <see langword="null" /> if no value is present.
+		/// </value>
+		/// <remarks>
+		///    This property is implemented using the "MUSICBRAINZ_ALBUMSTATUS" item.
+		///    http://musicbrainz.org/doc/PicardTagMapping
+		/// </remarks>
+		public override string MusicBrainzReleaseStatus {
+			get {return GetItemAsString ("MUSICBRAINZ_ALBUMSTATUS");}
+			set {SetValue ("MUSICBRAINZ_ALBUMSTATUS", value);}
+		}
+
+		/// <summary>
+		///    Gets and sets the MusicBrainz Release Type of the media
+		///    represented by the current instance.
+		/// </summary>
+		/// <value>
+		///    A <see cref="string" /> object containing the MusicBrainz
+		///    ReleaseType for the media represented by the current instance
+		///    or <see langword="null" /> if no value is present.
+		/// </value>
+		/// <remarks>
+		///    This property is implemented using the "MUSICBRAINZ_ALBUMTYPE" item.
+		///    http://musicbrainz.org/doc/PicardTagMapping
+		/// </remarks>
+		public override string MusicBrainzReleaseType {
+			get {return GetItemAsString ("MUSICBRAINZ_ALBUMTYPE");}
+			set {SetValue ("MUSICBRAINZ_ALBUMTYPE", value);}
+		}
+
+		/// <summary>
+		///    Gets and sets the MusicBrainz ReleaseCountry of the media
+		///    represented by the current instance.
+		/// </summary>
+		/// <value>
+		///    A <see cref="string" /> object containing the MusicBrainz
+		///    ReleaseCountry for the media represented by the current instance
+		///    or <see langword="null" /> if no value is present.
+		/// </value>
+		/// <remarks>
+		///    This property is implemented using the "RELEASECOUNTRY" item.
+		///    http://musicbrainz.org/doc/PicardTagMapping
+		/// </remarks>
+		public override string MusicBrainzReleaseCountry {
+			get {return GetItemAsString ("RELEASECOUNTRY");}
+			set {SetValue ("RELEASECOUNTRY", value);}
+		}
+
 		/// <summary>
 		///    Gets and sets a collection of pictures associated with
 		///    the media represented by the current instance.
@@ -1166,7 +1472,7 @@ namespace TagLib.Ape {
 				if (value == null || value.Length == 0)
 					return;
 				
-				foreach (Picture pic in value) {
+				foreach (IPicture pic in value) {
 					int type = (int) pic.Type;
 					
 					if (type >= picture_item_names.Length)
