@@ -537,6 +537,29 @@ namespace TagLib.Asf {
 			set {description.Title = value;}
 		}
 		
+		/// <summary>
+		///    Gets and sets the sort names for the Track Title of
+		///    the media described by the current instance.
+		/// </summary>
+		/// <value>
+		///    A <see cref="string" /> containing the sort name of 
+		///    the Track Title of the media described by the current
+		///    instance or null if no value is present.
+		/// </value>
+		/// <remarks>
+		///    This property is implemented using the "WM/TitleSortOrder"
+		///    field.
+		///    http://msdn.microsoft.com/en-us/library/aa386866(VS.85).aspx
+		/// </remarks>
+		public override string TitleSort {
+			get {
+				return GetDescriptorString ("WM/TitleSortOrder");
+			}
+			set {
+				SetDescriptorString (value, "WM/TitleSortOrder");
+			}
+		}
+		
 		/// <summary>
 		///    Gets and sets the performers or artists who performed in
 		///    the media described by the current instance.
@@ -556,6 +579,29 @@ namespace TagLib.Asf {
 			set {description.Author = string.Join ("; ", value);}
 		}
 		
+		/// <summary>
+		///    Gets and sets the sort names of the performers or artists
+		///    who performed in the media described by the current instance.
+		/// </summary>
+		/// <value>
+		///    A <see cref="string[]" /> containing the sort names for
+		///    the performers or artists who performed in the media
+		///    described by the current instance, or an empty array if
+		///    no value is present. 
+		/// </value>
+		/// <remarks>
+		///    This property is implemented using the "WM/ArtistSortOrder" field.
+		///    http://msdn.microsoft.com/en-us/library/aa386866(VS.85).aspx
+		/// </remarks>
+		public override string [] PerformersSort {
+			get {
+				return GetDescriptorStrings ("WM/ArtistSortOrder");
+			}
+			set {
+				SetDescriptorStrings (value, "WM/ArtistSortOrder");
+			}
+		}
+		
 		/// <summary>
 		///    Gets and sets the band or artist who is credited in the
 		///    creation of the entire album or collection containing the
@@ -582,6 +628,33 @@ namespace TagLib.Asf {
 			}
 		}
 		
+		/// <summary>
+		///    Gets and sets the sort names for the band or artist who
+		///    is credited in the creation of the entire album or
+		///    collection containing the media described by the
+		///    current instance.
+		/// </summary>
+		/// <value>
+		///    A <see cref="string[]" /> containing the sort names
+		///    for the band or artist who is credited in the creation
+		///    of the entire album or collection containing the media
+		///    described by the current instance or an empty array if
+		///    no value is present.
+		/// </value>
+		/// <remarks>
+		///    This property is implemented using the "WM/AlbumArtistSortOrder"
+		///    field.
+		///    http://msdn.microsoft.com/en-us/library/aa386866(VS.85).aspx
+		/// </remarks>
+		public override string [] AlbumArtistsSort {
+			get {
+				return GetDescriptorStrings ("WM/AlbumArtistSortOrder");
+			}
+			set {
+				SetDescriptorStrings (value, "WM/AlbumArtistSortOrder");
+			}
+		}
+		
 		/// <summary>
 		///    Gets and sets the composers of the media represented by
 		///    the current instance.
@@ -630,6 +703,29 @@ namespace TagLib.Asf {
 			}
 		}
 		
+		/// <summary>
+		///    Gets and sets the sort names for the Album Title of
+		///    the media described by the current instance.
+		/// </summary>
+		/// <value>
+		///    A <see cref="string" /> containing the sort name of 
+		///    the Album Title of the media described by the current
+		///    instance or null if no value is present.
+		/// </value>
+		/// <remarks>
+		///    This property is implemented using the "WM/AlbumSortOrder"
+		///    field.
+		///    http://msdn.microsoft.com/en-us/library/aa386866(VS.85).aspx
+		/// </remarks>
+		public override string AlbumSort {
+			get {
+				return GetDescriptorString ("WM/AlbumSortOrder");
+			}
+			set {
+				SetDescriptorString (value, "WM/AlbumSortOrder");
+			}
+		}
+		
 		/// <summary>
 		///    Gets and sets a user comment on the media represented by
 		///    the current instance.
@@ -1024,6 +1120,195 @@ namespace TagLib.Asf {
 			set {description.Copyright = value;}
 		}
 		
+		/// <summary>
+		///    Gets and sets the MusicBrainz Artist ID of
+		///    the media described by the current instance.
+		/// </summary>
+		/// <value>
+		///    A <see cref="string" /> containing the MusicBrainz 
+		///    ArtistID for the media described by the current
+		///    instance or null if no value is present.
+		/// </value>
+		/// <remarks>
+		///    This property is implemented using the "MusicBrainz/Artist Id"
+		///    field.
+		///    http://musicbrainz.org/doc/PicardTagMapping
+		/// </remarks>
+		public override string MusicBrainzArtistId {
+			get {return GetDescriptorString ("MusicBrainz/Artist Id");}
+			set {SetDescriptorString (value, "MusicBrainz/Artist Id");}
+		}
+
+		/// <summary>
+		///    Gets and sets the MusicBrainz Release ID of
+		///    the media described by the current instance.
+		/// </summary>
+		/// <value>
+		///    A <see cref="string" /> containing the MusicBrainz 
+		///    ReleaseID for the media described by the current
+		///    instance or null if no value is present.
+		/// </value>
+		/// <remarks>
+		///    This property is implemented using the "MusicBrainz/Album Id"
+		///    field.
+		///    http://musicbrainz.org/doc/PicardTagMapping
+		/// </remarks>
+		public override string MusicBrainzReleaseId {
+			get {return GetDescriptorString ("MusicBrainz/Album Id");}
+			set {SetDescriptorString (value, "MusicBrainz/Album Id");}
+		}
+
+		/// <summary>
+		///    Gets and sets the MusicBrainz Release Artist ID of
+		///    the media described by the current instance.
+		/// </summary>
+		/// <value>
+		///    A <see cref="string" /> containing the MusicBrainz 
+		///    ReleaseArtistID for the media described by the current
+		///    instance or null if no value is present.
+		/// </value>
+		/// <remarks>
+		///    This property is implemented using the "MusicBrainz/Album Artist Id"
+		///    field.
+		///    http://musicbrainz.org/doc/PicardTagMapping
+		/// </remarks>
+		public override string MusicBrainzReleaseArtistId {
+			get {return GetDescriptorString ("MusicBrainz/Album Artist Id");}
+			set {SetDescriptorString (value, "MusicBrainz/Album Artist Id");}
+		}
+
+		/// <summary>
+		///    Gets and sets the MusicBrainz Track ID of
+		///    the media described by the current instance.
+		/// </summary>
+		/// <value>
+		///    A <see cref="string" /> containing the MusicBrainz 
+		///    TrackID for the media described by the current
+		///    instance or null if no value is present.
+		/// </value>
+		/// <remarks>
+		///    This property is implemented using the "MusicBrainz/Track Id"
+		///    field.
+		///    http://musicbrainz.org/doc/PicardTagMapping
+		/// </remarks>
+		public override string MusicBrainzTrackId {
+			get {return GetDescriptorString ("MusicBrainz/Track Id");}
+			set {SetDescriptorString (value, "MusicBrainz/Track Id");}
+		}
+
+		/// <summary>
+		///    Gets and sets the MusicBrainz Disc ID of
+		///    the media described by the current instance.
+		/// </summary>
+		/// <value>
+		///    A <see cref="string" /> containing the MusicBrainz 
+		///    DiscID for the media described by the current
+		///    instance or null if no value is present.
+		/// </value>
+		/// <remarks>
+		///    This property is implemented using the "MusicBrainz/Disc Id"
+		///    field.
+		///    http://musicbrainz.org/doc/PicardTagMapping
+		/// </remarks>
+		public override string MusicBrainzDiscId {
+			get {return GetDescriptorString ("MusicBrainz/Disc Id");}
+			set {SetDescriptorString (value, "MusicBrainz/Disc Id");}
+		}
+
+		/// <summary>
+		///    Gets and sets the MusicIP PUID of
+		///    the media described by the current instance.
+		/// </summary>
+		/// <value>
+		///    A <see cref="string" /> containing the MusicIPPUID 
+		///    for the media described by the current instance or
+		///    null if no value is present.
+		/// </value>
+		/// <remarks>
+		///    This property is implemented using the "MusicIP/PUID"
+		///    field.
+		///    http://musicbrainz.org/doc/PicardTagMapping
+		/// </remarks>
+		public override string MusicIpId {
+			get {return GetDescriptorString ("MusicIP/PUID");}
+			set {SetDescriptorString (value, "MusicIP/PUID");}
+		}
+
+		// <summary>
+		//    Gets and sets the AmazonID of
+		//    the media described by the current instance.
+		// </summary>
+		// <value>
+		//    A <see cref="string" /> containing the AmazonID 
+		//    for the media described by the current instance or
+		//    null if no value is present.  
+		// </value>
+		// <remarks>
+		//    A definition on where to store the ASIN for
+		//    Windows Media is not currently defined
+		// </remarks>
+		//public override string AmazonId {
+		//    get { return null; }
+		//    set {}
+		//}
+
+		/// <summary>
+		///    Gets and sets the MusicBrainz Release Status of
+		///    the media described by the current instance.
+		/// </summary>
+		/// <value>
+		///    A <see cref="string" /> containing the MusicBrainz 
+		///    ReleaseStatus for the media described by the current
+		///    instance or null if no value is present.
+		/// </value>
+		/// <remarks>
+		///    This property is implemented using the "MusicBrainz/Album Status"
+		///    field.
+		///    http://musicbrainz.org/doc/PicardTagMapping
+		/// </remarks>
+		public override string MusicBrainzReleaseStatus {
+			get {return GetDescriptorString ("MusicBrainz/Album Status");}
+			set {SetDescriptorString (value, "MusicBrainz/Album Status");}
+		}
+
+		/// <summary>
+		///    Gets and sets the MusicBrainz Release Type of
+		///    the media described by the current instance.
+		/// </summary>
+		/// <value>
+		///    A <see cref="string" /> containing the MusicBrainz 
+		///    ReleaseType for the media described by the current
+		///    instance or null if no value is present.
+		/// </value>
+		/// <remarks>
+		///    This property is implemented using the "MusicBrainz/Album Type"
+		///    field.
+		///    http://musicbrainz.org/doc/PicardTagMapping
+		/// </remarks>
+		public override string MusicBrainzReleaseType {
+			get {return GetDescriptorString ("MusicBrainz/Album Type");}
+			set {SetDescriptorString (value, "MusicBrainz/Album Type");}
+		}
+
+		/// <summary>
+		///    Gets and sets the MusicBrainz Release Country of
+		///    the media described by the current instance.
+		/// </summary>
+		/// <value>
+		///    A <see cref="string" /> containing the MusicBrainz 
+		///    ReleaseCountry for the media described by the current
+		///    instance or null if no value is present.
+		/// </value>
+		/// <remarks>
+		///    This property is implemented using the "MusicBrainz/Album Release Country"
+		///    field.
+		///    http://musicbrainz.org/doc/PicardTagMapping
+		/// </remarks>
+		public override string MusicBrainzReleaseCountry {
+			get {return GetDescriptorString ("MusicBrainz/Album Release Country");}
+			set {SetDescriptorString (value, "MusicBrainz/Album Release Country");}
+		}
+
 		/// <summary>
 		///    Gets and sets a collection of pictures associated with
 		///    the media represented by the current instance.
