@@ -1104,6 +1104,46 @@ namespace TagLib {
 		
 #region Conversions
 		
+		/// <summary>
+		///    Converts an first four bytes of the current instance to
+		///    a <see cref="int" /> value.
+		/// </summary>
+		/// <param name="mostSignificantByteFirst">
+		///    <see langword="true" /> if the most significant byte
+		///    appears first (big endian format), or <see
+		///    langword="false" /> if the least significant byte appears
+		///    first (little endian format).
+		/// </param>
+		/// <returns>
+		///    A <see cref="int"/> value containing the value read from
+		///    the current instance.
+		/// </returns>
+		public int ToInt (bool mostSignificantByteFirst)
+		{
+			int ret = 0;
+			int last = Count > 4 ? 3 : Count - 1;
+
+			for (int i = 0; i <= last; i++) {
+				int offset = mostSignificantByteFirst ? last-i : i;
+				ret |= (int) this[i] << (offset * 8);
+			}
+
+			return ret;
+		}
+
+		/// <summary>
+		///    Converts an first four bytes of the current instance to
+		///    a <see cref="int" /> value using big-endian format.
+		/// </summary>
+		/// <returns>
+		///    A <see cref="int"/> value containing the value read from
+		///    the current instance.
+		/// </returns>
+		public int ToInt ()
+		{
+			return ToInt (true);
+		}
+
 		/// <summary>
 		///    Converts an first four bytes of the current instance to
 		///    a <see cref="uint" /> value.
@@ -1220,6 +1260,82 @@ namespace TagLib {
 		{
 			return ToULong (true);
 		}
+
+        /// <summary>
+        ///    Converts an first four bytes of the current instance to
+        ///    a <see cref="float" /> value.
+        /// </summary>
+        /// <param name="mostSignificantByteFirst">
+        ///    <see langword="true" /> if the most significant byte
+        ///    appears first (big endian format), or <see
+        ///    langword="false" /> if the least significant byte appears
+        ///    first (little endian format).
+        /// </param>
+        /// <returns>
+        ///    A <see cref="float"/> value containing the value read
+        ///    from the current instance.
+        /// </returns>
+        public float ToFloat (bool mostSignificantByteFirst)
+        {
+            byte [] bytes = (byte []) Data.Clone ();
+
+            if (mostSignificantByteFirst) {
+                Array.Reverse (bytes);
+            }
+
+            return BitConverter.ToSingle (bytes, 0);
+        }
+
+        /// <summary>
+        ///    Converts an first four bytes of the current instance to
+        ///    a <see cref="float" /> value using big-endian format.
+        /// </summary>
+        /// <returns>
+        ///    A <see cref="float"/> value containing the value read
+        ///    from the current instance.
+        /// </returns>
+        public float ToFloat ()
+        {
+            return ToFloat (true);
+        }
+
+        /// <summary>
+        ///    Converts an first eight bytes of the current instance to
+        ///    a <see cref="double" /> value.
+        /// </summary>
+        /// <param name="mostSignificantByteFirst">
+        ///    <see langword="true" /> if the most significant byte
+        ///    appears first (big endian format), or <see
+        ///    langword="false" /> if the least significant byte appears
+        ///    first (little endian format).
+        /// </param>
+        /// <returns>
+        ///    A <see cref="double"/> value containing the value read
+        ///    from the current instance.
+        /// </returns>
+        public double ToDouble (bool mostSignificantByteFirst)
+        {
+            byte [] bytes = (byte []) Data.Clone ();
+
+            if (mostSignificantByteFirst) {
+                Array.Reverse (bytes);
+            }
+
+            return BitConverter.ToDouble (bytes, 0);
+        }
+
+        /// <summary>
+        ///    Converts an first eight bytes of the current instance to
+        ///    a <see cref="double" /> value using big-endian format.
+        /// </summary>
+        /// <returns>
+        ///    A <see cref="double"/> value containing the value read
+        ///    from the current instance.
+        /// </returns>
+        public double ToDouble ()
+        {
+            return ToDouble (true);
+        }
 		
 		/// <summary>
 		///    Converts a portion of the current instance to a <see
@@ -1256,7 +1372,7 @@ namespace TagLib {
 				throw new ArgumentOutOfRangeException ("count");
 			
 			ByteVector bom = type == StringType.UTF16 &&
-				this.data.Count > 1 ? Mid (offset, 2) : null;
+				this.data.Count - offset > 1 ? Mid (offset, 2) : null;
 			
 			string s = StringTypeToEncoding (type, bom)
 				.GetString (Data, offset, count);
@@ -1665,6 +1781,49 @@ namespace TagLib {
 		
 #region Static Conversions
 		
+		/// <summary>
+		///    Converts a value into a data representation.
+		/// </summary>
+		/// <param name="value">
+		///    A <see cref="int"/> value to convert into bytes.
+		/// </param>
+		/// <param name="mostSignificantByteFirst">
+		///    <see langword="true" /> if the most significant byte is
+		///    to appear first (big endian format), or <see
+		///    langword="false" /> if the least significant byte is to
+		///    appear first (little endian format).
+		/// </param>
+		/// <returns>
+		///    A <see cref="ByteVector"/> object containing the encoded
+		///    representation of <paramref name="value" />.
+		/// </returns>
+		public static ByteVector FromInt (int value,
+		                                   bool mostSignificantByteFirst)
+		{
+			ByteVector vector = new ByteVector();
+			for(int i = 0; i < 4; i++) {
+				int offset = mostSignificantByteFirst ? 3-i : i;
+				vector.Add ((byte)(value >> (offset * 8) & 0xFF));
+			}
+
+			return vector;
+		}
+
+		/// <summary>
+		///    Converts an value into a big-endian data representation.
+		/// </summary>
+		/// <param name="value">
+		///    A <see cref="int"/> value to convert into bytes.
+		/// </param>
+		/// <returns>
+		///    A <see cref="ByteVector"/> object containing the encoded
+		///    representation of <paramref name="value" />.
+		/// </returns>
+		public static ByteVector FromInt (int value)
+		{
+			return FromInt (value, true);
+		}
+
 		/// <summary>
 		///    Converts an unsigned value into a data representation.
 		/// </summary>
