@@ -1,12 +1,14 @@
 //
 // Tag.cs: Provide support for reading and writing ID3v2 tags.
 //
-// Author:
+// Authors:
 //   Brian Nickel (brian.nickel@gmail.com)
+//   Gabriel BUrt (gabriel.burt@gmail.com)
 //
 // Original Source:
 //   id3v2tag.cpp from TagLib
 //
+// Copyright (C) 2010 Novell, Inc.
 // Copyright (C) 2005-2007 Brian Nickel
 // Copyright (C) 2002,2003 Scott Wheeler (Original Implementation)
 //
@@ -555,7 +557,9 @@ namespace TagLib.Id3v2 {
 			
 			bool has_footer = (header.Flags &
 				HeaderFlags.FooterPresent) != 0;
-			
+			bool unsynchAtFrameLevel = (header.Flags & HeaderFlags.Unsynchronisation) != 0 && Version >= 4;
+			bool unsynchAtTagLevel = (header.Flags & HeaderFlags.Unsynchronisation) != 0 && Version < 4;
+
 			header.MajorVersion = has_footer ? (byte) 4 : Version;
 			
 			ByteVector tag_data = new ByteVector ();
@@ -566,6 +570,9 @@ namespace TagLib.Id3v2 {
 			// Loop through the frames rendering them and adding
 			// them to the tag_data.
 			foreach (Frame frame in frame_list) {
+				if (unsynchAtFrameLevel)
+					frame.Flags |= FrameFlags.Unsynchronisation;
+
 				if ((frame.Flags &
 					FrameFlags.TagAlterPreservation) != 0)
 					continue;
@@ -578,7 +585,7 @@ namespace TagLib.Id3v2 {
 			}
 			
 			// Add unsyncronization bytes if necessary.
-			if ((header.Flags & HeaderFlags.Unsynchronisation) != 0)
+			if (unsynchAtTagLevel)
 				SynchData.UnsynchByteVector (tag_data);
 			
 			// Compute the amount of padding, and append that to
@@ -781,7 +788,7 @@ namespace TagLib.Id3v2 {
 		/// <exception cref="ArgumentNullException">
 		///    <paramref name="file" /> is <see langword="null" />.
 		/// </exception>
-		/// <exception name="ArgumentOutOfRangeException">
+		/// <exception cref="ArgumentOutOfRangeException">
 		///    <paramref name="position" /> is less than 0 or greater
 		///    than the size of the file.
 		/// </exception>
@@ -826,11 +833,11 @@ namespace TagLib.Id3v2 {
 		{
 			if (data == null)
 				throw new ArgumentNullException ("data");
-			
-			// If the entire tag is marked as unsynchronized,
-			// resynchronize it.
-			
-			if ((header.Flags & HeaderFlags.Unsynchronisation) != 0)
+
+			// If the entire tag is marked as unsynchronized, and this tag
+			// is version id3v2.3 or lower, resynchronize it.
+			bool fullTagUnsynch =  (header.MajorVersion < 4) && ((header.Flags & HeaderFlags.Unsynchronisation) != 0);
+			if (fullTagUnsynch)
 				SynchData.ResynchByteVector (data);
 			
 			int frame_data_position = 0;
@@ -854,6 +861,7 @@ namespace TagLib.Id3v2 {
 			// for post-processing, so check for them as they are
 			// loaded.
 			TextInformationFrame tdrc = null;
+			TextInformationFrame tyer = null;
 			TextInformationFrame tdat = null;
 			TextInformationFrame time = null;
 			
@@ -865,20 +873,24 @@ namespace TagLib.Id3v2 {
 				// frame data.
 				if(data [frame_data_position] == 0)
 					break;
-				
+
 				Frame frame = null;
-				
+
 				try {
-					frame = FrameFactory.CreateFrame (data,
+					frame = FrameFactory.CreateFrame(
+						data,
 						ref frame_data_position,
-						header.MajorVersion);
+						header.MajorVersion,
+						fullTagUnsynch);
 				} catch (NotImplementedException) {
 					continue;
+				} catch (CorruptFileException) {
+					continue;
 				}
-				
+
 				if(frame == null)
 					break;
-				
+
 				// Only add frames that contain data.
 				if (frame.Size == 0)
 					continue;
@@ -896,6 +908,9 @@ namespace TagLib.Id3v2 {
 				if (tdrc == null &&
 					frame.FrameId.Equals (FrameType.TDRC)) {
 					tdrc = frame as TextInformationFrame;
+				} else if (tyer == null &&
+					frame.FrameId.Equals (FrameType.TYER)) {
+					tyer = frame as TextInformationFrame;
 				} else if (tdat == null &&
 					frame.FrameId.Equals (FrameType.TDAT)) {
 					tdat = frame as TextInformationFrame;
@@ -904,52 +919,44 @@ namespace TagLib.Id3v2 {
 					time = frame as TextInformationFrame;
 				}
 			}
-			
-			// Post-processing: Combine the three frames into one
-			// TDRC frame.
-			if (tdrc == null || tdat == null) {
-				if (tdat != null)
-					RemoveFrames (FrameType.TDAT);
-				
-				if (time != null)
-					RemoveFrames (FrameType.TIME);
-				
+
+			// Try to fill out the date/time of the TDRC frame.  Can't do that if no TDRC
+			// frame exists, or if there is no TDAT frame, or if TDRC already has the date.
+			if (tdrc == null || tdat == null || tdrc.ToString ().Length > 4) {
 				return;
 			}
-			
-			StringBuilder tdrc_text = new StringBuilder (
-				tdrc.ToString ());
-			
-			string tdat_text = tdat.ToString ();
-			
-			if (tdrc_text.Length != 4 || tdat_text.Length != 4) {
+
+			string year = tdrc.ToString ();
+			if (year.Length != 4)
+				return;
+
+			// Start with the year already in TDRC, then add the TDAT and TIME if available
+			StringBuilder tdrc_text = new StringBuilder ();
+			tdrc_text.Append (year);
+
+			// Add the date
+			if (tdat != null) {
+				string tdat_text = tdat.ToString ();
+				if (tdat_text.Length == 4) {
+					tdrc_text.Append ("-").Append (tdat_text, 0, 2)
+						.Append ("-").Append (tdat_text, 2, 2);
+
+					// Add the time
+					if (time != null) {
+						string time_text = time.ToString ();
+							
+						if (time_text.Length == 4)
+							tdrc_text.Append ("T").Append (time_text, 0, 2)
+								.Append (":").Append (time_text, 2, 2);
+
+						RemoveFrames (FrameType.TIME);
+					}
+				}
+
 				RemoveFrames (FrameType.TDAT);
-				
-				if (time != null)
-					RemoveFrames (FrameType.TIME);
-				
-				return;
 			}
-			
-			tdrc_text.Append ("-").Append (tdat_text, 0, 2)
-				.Append ("-").Append (tdat_text, 2, 2);
-			
-			RemoveFrames (FrameType.TDAT);
-			
-			if (time == null) {
-				tdrc.Text = new string [] {tdrc_text.ToString ()};
-				return;
-			}
-			
-			string time_text = time.ToString ();
-				
-			if (time_text.Length == 4)
-				tdrc_text.Append ("T").Append (time_text, 0, 2)
-					.Append (":").Append (time_text, 2, 2);
-					
-			tdrc.Text = new string [] {tdrc_text.ToString ()};
-			
-			RemoveFrames (FrameType.TIME);
+
+			tdrc.Text = new string [] { tdrc_text.ToString () };
 		}
 		
 #endregion
@@ -1039,7 +1046,86 @@ namespace TagLib.Id3v2 {
 			
 			return 0;
 		}
-		
+
+		/// <summary>
+		/// Gets a TXXX frame via reference of the description field
+		/// </summary>
+		/// <param name="description">String containing the description field</param>
+		/// <returns>UserTextInformationFrame (TXXX) that corresponds to the description</returns>
+		private string GetUserTextAsString (string description){
+
+			//Gets the TXXX frame, frame will be null if nonexistant
+			UserTextInformationFrame frame = UserTextInformationFrame.Get (
+				this, description, false);
+
+			//TXXX frames support multivalue strings, join them up and return
+			//only the text from the frame.
+			string result = frame == null ? null : string.Join (";",frame.Text);
+			return string.IsNullOrEmpty (result) ? null : result;
+
+		}
+
+		/// <summary>
+		/// Creates and/or sets a UserTextInformationFrame (TXXX)  with the given
+		/// description and text.
+		/// </summary>
+		/// <param name="description">String containing the Description field for the
+		/// TXXX frame</param>
+		/// <param name="text">String containing the Text field for the TXXX frame</param>
+		private void SetUserTextAsString(string description, string text) {
+
+			//Get the TXXX frame, create a new one if needed
+			UserTextInformationFrame frame = UserTextInformationFrame.Get(
+				this, description, true);
+
+			if (!string.IsNullOrEmpty(text)) {
+				frame.Text = text.Split(';');
+			}
+			else {
+			//Text string is null or empty, delete the frame, prevent empties
+				RemoveFrame(frame);
+			}
+
+		}
+
+		/// <summary>
+		/// Gets the text from a particular UFID frame, referenced by the owner field
+		/// </summary>
+		/// <param name="owner">String containing the "Owner" data</param>
+		/// <returns>String containing the text from the UFID frame, or null</returns>
+		private string GetUfidText(string owner) {
+
+			//Get the UFID frame, frame will be null if nonexistant
+			UniqueFileIdentifierFrame frame = UniqueFileIdentifierFrame.Get(
+				this, owner, false);
+			
+			//If the frame existed: frame.Identifier is a bytevector, get a string
+			string result = frame == null ? null : frame.Identifier.ToString();
+			return string.IsNullOrEmpty (result) ? null : result;
+		}
+
+		/// <summary>
+		/// Creates and/or sets the text for a UFID frame, referenced by owner
+		/// </summary>
+		/// <param name="owner">String containing the Owner field</param>
+		/// <param name="text">String containing the text to set for the frame</param>
+		private void SetUfidText(string owner, string text) {
+
+			//Get a UFID frame, create if necessary
+			UniqueFileIdentifierFrame frame = UniqueFileIdentifierFrame.Get(
+				this, owner, true);
+
+			//If we have a real string, convert to ByteVector and apply to frame
+			if (!string.IsNullOrEmpty(text)) {
+				ByteVector identifier = ByteVector.FromString(text, StringType.UTF8);
+				frame.Identifier = identifier;
+			}
+			else {
+				//String was null or empty, remove the frame to prevent empties
+				RemoveFrame(frame);
+			}
+		}
+
 		/// <summary>
 		///    Moves a specified frame so it is the first of its type in
 		///    the tag.
@@ -1129,6 +1215,24 @@ namespace TagLib.Id3v2 {
 			set {SetTextFrame (FrameType.TIT2, value);}
 		}
 		
+		/// <summary>
+		///    Gets and sets the sort names of the Title of the
+		///    media represented by the current instance.
+		/// </summary>
+		/// <value>
+		///    A <see cref="string" /> containing the sort names for
+		///    the Title of the media described by the current instance,
+		///    or null if no value is present. 
+		/// </value>
+		/// <remarks>
+		///    This property is implemented using the "TSOT" Text
+		///    Information Frame.
+		/// </remarks>
+		public override string TitleSort {
+			get {return GetTextAsString (FrameType.TSOT);}
+			set {SetTextFrame (FrameType.TSOT, value);}
+		}
+
 		/// <summary>
 		///    Gets and sets the performers or artists who performed in
 		///    the media described by the current instance.
@@ -1148,6 +1252,45 @@ namespace TagLib.Id3v2 {
 			set {SetTextFrame (FrameType.TPE1, value);}
 		}
 		
+		/// <summary>
+		///    Gets and sets the sort names of the performers or artists
+		///    who performed in the media described by the current instance.
+		/// </summary>
+		/// <value>
+		///    A <see cref="string[]" /> containing the sort names for
+		///    the performers or artists who performed in the media
+		///    described by the current instance, or an empty array if
+		///    no value is present. 
+		/// </value>
+		/// <remarks>
+		///    This property is implemented using the "TSOP" Text
+		///    Information Frame. http://www.id3.org/id3v2.4.0-frames
+		/// </remarks>
+		public override string [] PerformersSort {
+			get {return GetTextAsArray (FrameType.TSOP);}
+			set {SetTextFrame (FrameType.TSOP, value);}
+		}
+
+		/// <summary>
+		///    Gets and sets the sort names of the band or artist who is 
+		///    credited in the creation of the entire album or collection
+		///    containing the media described by the current instance.
+		/// </summary>
+		/// <value>
+		///    A <see cref="string[]" /> containing the sort names for
+		///    the performers or artists who performed in the media
+		///    described by the current instance, or an empty array if
+		///    no value is present. 
+		/// </value>
+		/// <remarks>
+		///    This property is implemented using the "TSO2" Text
+		///    Information Frame. http://www.id3.org/iTunes
+		/// </remarks>
+		public override string [] AlbumArtistsSort {
+			get {return GetTextAsArray (FrameType.TSO2);}
+			set {SetTextFrame (FrameType.TSO2, value);}
+		}
+
 		/// <summary>
 		///    Gets and sets the band or artist who is credited in the
 		///    creation of the entire album or collection containing the
@@ -1186,6 +1329,25 @@ namespace TagLib.Id3v2 {
 			set {SetTextFrame (FrameType.TCOM, value);}
 		}
 		
+		/// <summary>
+		///    Gets and sets the sort names of the composers of the
+		///    media represented by the current instance.
+		/// </summary>
+		/// <value>
+		///    A <see cref="string[]" /> containing the sort names for
+		///    the performers or artists who performed in the media
+		///    described by the current instance, or an empty array if
+		///    no value is present. 
+		/// </value>
+		/// <remarks>
+		///    This property is implemented using the "TSOC" Text
+		///    Information Frame. http://www.id3.org/id3v2.4.0-frames
+		/// </remarks>
+		public override string [] ComposersSort {
+			get {return GetTextAsArray (FrameType.TSOC);}
+			set {SetTextFrame (FrameType.TSOC, value);}
+		}
+
 		/// <summary>
 		///    Gets and sets the album of the media represented by the
 		///    current instance.
@@ -1204,6 +1366,24 @@ namespace TagLib.Id3v2 {
 			set {SetTextFrame (FrameType.TALB, value);}
 		}
 		
+		/// <summary>
+		///    Gets and sets the sort names of the Album title of the
+		///    media represented by the current instance.
+		/// </summary>
+		/// <value>
+		///    A <see cref="string" /> containing the sort names for
+		///    the Title in the media described by the current instance,
+		///    or null if no value is present. 
+		/// </value>
+		/// <remarks>
+		///    This property is implemented using the "TSOA" Text
+		///    Information Frame. http://www.id3.org/id3v2.4.0-frames
+		/// </remarks>
+		public override string AlbumSort {
+			get {return GetTextAsString (FrameType.TSOA);}
+			set {SetTextFrame (FrameType.TSOA, value);}
+		}
+		
 		/// <summary>
 		///    Gets and sets a user comment on the media represented by
 		///    the current instance.
@@ -1545,6 +1725,176 @@ namespace TagLib.Id3v2 {
 			get {return GetTextAsString (FrameType.TCOP);}
 			set {SetTextFrame (FrameType.TCOP, value);}
 		}
+
+		/// <summary>
+		///    Gets and sets the MusicBrainz ArtistID
+		/// </summary>
+		/// <value>
+		///    A <see cref="string" /> containing the MusicBrainz
+		///    ArtistID for the media described by the current 
+		///    instance, or null if no value is present. 
+		/// </value>
+		/// <remarks>
+		///    This property is implemented using the "TXXX:MusicBrainz Artist Id" frame.
+		///    http://musicbrainz.org/doc/PicardTagMapping
+		/// </remarks>
+		public override string MusicBrainzArtistId {
+			get {return GetUserTextAsString ("MusicBrainz Artist Id");}
+			set {SetUserTextAsString ("MusicBrainz Artist Id",value);}
+		}
+
+		/// <summary>
+		///    Gets and sets the MusicBrainz ReleaseID
+		/// </summary>
+		/// <value>
+		///    A <see cref="string" /> containing the MusicBrainz
+		///    ReleaseID for the media described by the current 
+		///    instance, or null if no value is present. 
+		/// </value>
+		/// <remarks>
+		///    This property is implemented using the "TXXX:MusicBrainz Album Id" frame.
+		///    http://musicbrainz.org/doc/PicardTagMapping
+		/// </remarks>
+		public override string MusicBrainzReleaseId {
+			get {return GetUserTextAsString ("MusicBrainz Album Id");}
+			set {SetUserTextAsString ("MusicBrainz Album Id",value);}
+		}
+
+		/// <summary>
+		///    Gets and sets the MusicBrainz ReleaseArtistID
+		/// </summary>
+		/// <value>
+		///    A <see cref="string" /> containing the MusicBrainz
+		///    ReleaseArtistID for the media described by the current 
+		///    instance, or null if no value is present. 
+		/// </value>
+		/// <remarks>
+		///    This property is implemented using the "TXXX:MusicBrainz Album Artist Id" frame.
+		///    http://musicbrainz.org/doc/PicardTagMapping
+		/// </remarks>
+		public override string MusicBrainzReleaseArtistId {
+			get {return GetUserTextAsString ("MusicBrainz Album Artist Id");}
+			set {SetUserTextAsString ("MusicBrainz Album Artist Id",value);}
+		}
+		
+		/// <summary>
+		///    Gets and sets the MusicBrainz TrackID
+		/// </summary>
+		/// <value>
+		///    A <see cref="string" /> containing the MusicBrainz
+		///    TrackID for the media described by the current 
+		///    instance, or null if no value is present. 
+		/// </value>
+		/// <remarks>
+		///    This property is implemented using the "UFID:http://musicbrainz.org" frame.
+		///    http://musicbrainz.org/doc/PicardTagMapping
+		/// </remarks>
+		public override string MusicBrainzTrackId {
+		    get { return GetUfidText ("http://musicbrainz.org");}
+		    set {SetUfidText ("http://musicbrainz.org", value);}
+		}
+
+		/// <summary>
+		///    Gets and sets the MusicBrainz DiscID
+		/// </summary>
+		/// <value>
+		///    A <see cref="string" /> containing the MusicBrainz
+		///    DiscID for the media described by the current 
+		///    instance, or null if no value is present. 
+		/// </value>
+		/// <remarks>
+		///    This property is implemented using the "TXXX:MusicBrainz Disc Id" frame.
+		///    http://musicbrainz.org/doc/PicardTagMapping
+		/// </remarks>
+		public override string MusicBrainzDiscId {
+			get {return GetUserTextAsString ("MusicBrainz Disc Id");}
+			set {SetUserTextAsString ("MusicBrainz Disc Id",value);}
+		}
+
+		/// <summary>
+		///    Gets and sets the MusicIP PUID
+		/// </summary>
+		/// <value>
+		///    A <see cref="string" /> containing the MusicIP PUID
+		///    for the media described by the current 
+		///    instance, or null if no value is present. 
+		/// </value>
+		/// <remarks>
+		///    This property is implemented using the "TXXX:MusicIP PUID" frame.
+		///    http://musicbrainz.org/doc/PicardTagMapping
+		/// </remarks>
+		public override string MusicIpId {
+			get {return GetUserTextAsString ("MusicIP PUID");}
+			set {SetUserTextAsString ("MusicIP PUID",value);}
+		}
+
+		/// <summary>
+		///    Gets and sets the Amazon ID (ASIN)
+		/// </summary>
+		/// <value>
+		///    A <see cref="string" /> containing the Amazon Id
+		///    for the media described by the current 
+		///    instance, or null if no value is present. 
+		/// </value>
+		/// <remarks>
+		///    This property is implemented using the "TXXX:ASIN" frame.
+		///    http://musicbrainz.org/doc/PicardTagMapping
+		/// </remarks>
+		public override string AmazonId {
+			get {return GetUserTextAsString ("ASIN");}
+			set {SetUserTextAsString ("ASIN",value);}
+		}
+
+		/// <summary>
+		///    Gets and sets the MusicBrainz ReleaseStatus
+		/// </summary>
+		/// <value>
+		///    A <see cref="string" /> containing the MusicBrainz
+		///    ReleaseStatus for the media described by the current 
+		///    instance, or null if no value is present. 
+		/// </value>
+		/// <remarks>
+		///    This property is implemented using the "TXXX:MusicBrainz Album Status" frame.
+		///    http://musicbrainz.org/doc/PicardTagMapping
+		/// </remarks>
+		public override string MusicBrainzReleaseStatus {
+			get {return GetUserTextAsString ("MusicBrainz Album Status");}
+			set {SetUserTextAsString ("MusicBrainz Album Status",value);}
+		}
+
+		/// <summary>
+		///    Gets and sets the MusicBrainz ReleaseType
+		/// </summary>
+		/// <value>
+		///    A <see cref="string" /> containing the MusicBrainz
+		///    ReleaseType for the media described by the current 
+		///    instance, or null if no value is present. 
+		/// </value>
+		/// <remarks>
+		///    This property is implemented using the "TXXX:MusicBrainz Album Type" frame.
+		///    http://musicbrainz.org/doc/PicardTagMapping
+		/// </remarks>
+		public override string MusicBrainzReleaseType {
+			get {return GetUserTextAsString ("MusicBrainz Album Type");}
+			set {SetUserTextAsString ("MusicBrainz Album Type",value);}
+		}
+
+		/// <summary>
+		///    Gets and sets the MusicBrainz ReleaseCountry
+		/// </summary>
+		/// <value>
+		///    A <see cref="string" /> containing the MusicBrainz
+		///    ReleaseCountry for the media described by the current 
+		///    instance, or null if no value is present. 
+		/// </value>
+		/// <remarks>
+		///    This property is implemented using the "TXXX:MusicBrainz Album Release Country" frame.
+		///    http://musicbrainz.org/doc/PicardTagMapping
+		/// </remarks>
+		public override string MusicBrainzReleaseCountry {
+			get {return GetUserTextAsString ("MusicBrainz Album Release Country");}
+			set {SetUserTextAsString ("MusicBrainz Album Release Country",value);}
+		}
 		
 		/// <summary>
 		///    Gets and sets a collection of pictures associated with
