@@ -406,6 +406,132 @@ namespace TagLib.Mpeg4 {
 			meta_box.RemoveChild (ilst_box);
 		}
 		
+		/// <summary>
+		/// Gets the text string from a specific data box in a Dash (----) atom
+		/// </summary>
+		/// <param name="meanstring">String specifying text from mean box</param>
+		/// <param name="namestring">String specifying text from name box</param>
+		/// <returns>Text string from data box</returns>
+		public string GetDashBox(string meanstring, string namestring)
+		{
+			AppleDataBox data_box = GetDashAtoms(meanstring, namestring);
+			if (data_box != null) {
+				return data_box.Text;
+			} else {
+				return null;
+			}
+		}
+			
+		/// <summary>
+		/// Sets a specific strings in Dash (----) atom.  This method updates
+		/// and existing atom, or creates a new one.  If an empty datastring is
+		/// specified, the Dash box and its children are removed.
+		/// </summary>
+		/// <param name="meanstring">String specifying text for mean box</param>
+		/// <param name="namestring">String specifying text for name box</param>
+		/// <param name="datastring">String specifying text for data box</param>
+		public void SetDashBox(string meanstring, string namestring, string datastring)
+		{
+			AppleDataBox data_box = GetDashAtoms(meanstring, namestring);
+			
+			// If we did find a data_box and we have an empty datastring we should
+			// remove the entire dash box.
+			if (data_box != null && string.IsNullOrEmpty(datastring)) {
+				AppleAnnotationBox dash_box = GetParentDashBox(meanstring, namestring);
+				dash_box.ClearChildren();
+				ilst_box.RemoveChild(dash_box);
+				return;
+			}
+			
+			if (data_box != null) {
+				data_box.Text = datastring;
+			} else {
+				//Create the new boxes, should use 1 for text as a flag
+				AppleAdditionalInfoBox amean_box = new AppleAdditionalInfoBox(BoxType.Mean);
+				AppleAdditionalInfoBox aname_box = new AppleAdditionalInfoBox(BoxType.Name);
+				AppleDataBox adata_box = new AppleDataBox(BoxType.Data, 1);
+				amean_box.Text = meanstring;
+				aname_box.Text = namestring;
+				adata_box.Text = datastring;
+				AppleAnnotationBox whole_box = new AppleAnnotationBox(BoxType.DASH);
+				whole_box.AddChild(amean_box);
+				whole_box.AddChild(aname_box);
+				whole_box.AddChild(adata_box);
+				ilst_box.AddChild(whole_box);
+			}
+		}
+		
+		/// <summary>
+		/// Gets the AppleDataBox that corresponds to the specified mean and name values.
+		/// </summary>
+		/// <param name="meanstring">String specifying text for mean box</param>
+		/// <param name="namestring">String specifying text for name box</param>
+		/// <returns>Existing AppleDataBox or null if one does not exist</returns>
+		private AppleDataBox GetDashAtoms(string meanstring, string namestring)
+		{
+			foreach (Box box in ilst_box.Children) {
+				if (box.BoxType != BoxType.DASH)
+					continue;
+				
+				// Get the mean and name boxes, make sure
+				// they're legit, check the Text fields for
+				// a match.  If we have a match return
+				// the AppleDatabox containing the data
+				
+				AppleAdditionalInfoBox mean_box =
+					(AppleAdditionalInfoBox)
+					box.GetChild(BoxType.Mean);
+				AppleAdditionalInfoBox name_box =
+					(AppleAdditionalInfoBox)
+					box.GetChild(BoxType.Name);
+					
+				if (mean_box == null || name_box == null ||
+					mean_box.Text != meanstring ||
+					name_box.Text != namestring) {
+					continue;
+				} else {
+					return (AppleDataBox)box.GetChild(BoxType.Data);
+				}
+			}
+			// If we haven't returned the found box yet, there isn't one, return null
+			return null;
+		}
+		
+		/// <summary>
+		/// Returns the Parent Dash box object for a given mean/name combination
+		/// </summary>
+		/// <param name="meanstring">String specifying text for mean box</param>
+		/// <param name="namestring">String specifying text for name box</param>
+		/// <returns>AppleAnnotationBox object that is the parent for the mean/name combination</returns>
+		private AppleAnnotationBox GetParentDashBox(string meanstring, string namestring)
+		{
+			foreach (Box box in ilst_box.Children) {
+				if (box.BoxType != BoxType.DASH)
+					continue;
+				
+				// Get the mean and name boxes, make sure
+				// they're legit, check the Text fields for
+				// a match.  If we have a match return
+				// the AppleAnnotationBox that is the Parent
+				
+				AppleAdditionalInfoBox mean_box =
+					(AppleAdditionalInfoBox)
+					box.GetChild(BoxType.Mean);
+				AppleAdditionalInfoBox name_box =
+					(AppleAdditionalInfoBox)
+					box.GetChild(BoxType.Name);
+					
+				if (mean_box == null || name_box == null ||
+					mean_box.Text != meanstring ||
+					name_box.Text != namestring) {
+					continue;
+				} else {
+					return (AppleAnnotationBox)box;
+				}
+			}
+			// If we haven't returned the found box yet, there isn't one, return null
+			return null;
+		}
 		#endregion
 		
 		
@@ -908,11 +1034,11 @@ namespace TagLib.Mpeg4 {
 				}
 				
 				SetData (BoxType.Tmpo,
-					ByteVector.FromUInt (value),
+					ByteVector.FromUShort ((ushort)value),
 					(uint) AppleDataBox.FlagType.ForTempo);
 			}
 		}
-      
+		
 		/// <summary>
 		///    Gets and sets the conductor or director of the media
 		///    represented by the current instance.
@@ -957,6 +1083,283 @@ namespace TagLib.Mpeg4 {
 			set {SetText(BoxType.Cprt, value);}
 		}
 		
+		/// <summary>
+		///    Gets and sets the sort names for the band or artist who
+		///    is credited in the creation of the entire album or
+		///    collection containing the media described by the
+		///    current instance.
+		/// </summary>
+		/// <value>
+		///    A <see cref="string[]" /> containing the sort names
+		///    for the band or artist who is credited in the creation
+		///    of the entire album or collection containing the media
+		///    described by the current instance or an empty array if
+		///    no value is present.
+		/// </value>
+		/// <remarks>
+		///    This property is implemented using the "soaa"
+		///    Box type.
+		///    http://musicbrainz.org/doc/PicardTagMapping
+		///    http://code.google.com/p/mp4v2/wiki/iTunesMetadata
+		/// </remarks>
+		public override string [] AlbumArtistsSort {
+			get {return GetText (BoxType.Soaa);}
+			set {SetText (BoxType.Soaa, value);}
+		}
+		
+		/// <summary>
+		///    Gets and sets the sort names of the performers or artists
+		///    who performed in the media described by the current instance.
+		/// </summary>
+		/// <value>
+		///    A <see cref="string[]" /> containing the sort names for
+		///    the performers or artists who performed in the media
+		///    described by the current instance, or an empty array if
+		///    no value is present. 
+		/// </value>
+		/// <remarks>
+		///    This property is implemented using the "soar" box type.
+		///    http://musicbrainz.org/doc/PicardTagMapping
+		///    http://code.google.com/p/mp4v2/wiki/iTunesMetadata
+		/// </remarks>
+		public override string[] PerformersSort {
+			get {return GetText (BoxType.Soar);}
+			set {SetText (BoxType.Soar, value);}
+		}
+		
+		/// <summary>
+		///    Gets and sets the sort names of the Composer credited
+		///    in the media described by the current instance.
+		/// </summary>
+		/// <value>
+		///    A <see cref="string[]" /> containing the sort names for
+		///    the Composers in the media described by the current instance,
+		///    or an empty array if no value is present. 
+		/// </value>
+		/// <remarks>
+		///    This property is implemented using the "soar" box type.
+		///    http://musicbrainz.org/doc/PicardTagMapping
+		///    http://code.google.com/p/mp4v2/wiki/iTunesMetadata
+		/// </remarks>
+		public override string[] ComposersSort {
+			get {return GetText (BoxType.Soco);}
+			set {SetText (BoxType.Soco, value);}
+		}
+		
+		/// <summary>
+		///    Gets and sets the sort names of the Album Title of
+		///    the media described by the current instance.
+		/// </summary>
+		/// <value>
+		///    A <see cref="string" /> containing the sort names for
+		///    the Album Title in the media described by the current
+		///    instance, or null if no value is present. 
+		/// </value>
+		/// <remarks>
+		///    This property is implemented using the "soal" box type.
+		///    http://musicbrainz.org/doc/PicardTagMapping
+		///    http://code.google.com/p/mp4v2/wiki/iTunesMetadata
+		/// </remarks>
+		public override string AlbumSort {
+			get {
+				string [] text = GetText (BoxType.Soal);
+				return text.Length == 0 ? null : text [0];
+			}
+			set {SetText (BoxType.Soal, value);}
+		}
+		
+		/// <summary>
+		///    Gets and sets the sort names of the Track Title in the
+		///    media described by the current instance.
+		/// </summary>
+		/// <value>
+		///    A <see cref="string" /> containing the sort names for
+		///    the Track Title in the media described by the current 
+		///    instance, or null if no value is present. 
+		/// </value>
+		/// <remarks>
+		///    This property is implemented using the "sonm" box type.
+		///    http://musicbrainz.org/doc/PicardTagMapping
+		///    http://code.google.com/p/mp4v2/wiki/iTunesMetadata
+		/// </remarks>
+		public override string TitleSort {
+			get {
+				string [] text = GetText (BoxType.Sonm);
+				return text.Length == 0 ? null : text [0];
+			}
+			set {SetText (BoxType.Sonm, value);}
+		}
+
+		/// <summary>
+		///    Gets and sets the MusicBrainz ArtistID
+		/// </summary>
+		/// <value>
+		///    A <see cref="string" /> containing the MusicBrainz
+		///    ArtistID for the media described by the current 
+		///    instance, or null if no value is present. 
+		/// </value>
+		/// <remarks>
+		///    This property is implemented using the "dash"/"----" box type.
+		///    http://musicbrainz.org/doc/PicardTagMapping
+		/// </remarks>
+		public override string MusicBrainzArtistId {
+			get {return GetDashBox("com.apple.iTunes","MusicBrainz Artist Id");}
+			set {SetDashBox("com.apple.iTunes", "MusicBrainz Artist Id", value);}
+		}
+
+		/// <summary>
+		///    Gets and sets the MusicBrainz ReleaseID
+		/// </summary>
+		/// <value>
+		///    A <see cref="string" /> containing the MusicBrainz
+		///    ReleaseID for the media described by the current 
+		///    instance, or null if no value is present. 
+		/// </value>
+		/// <remarks>
+		///    This property is implemented using the "dash"/"----" box type.
+		///    http://musicbrainz.org/doc/PicardTagMapping
+		/// </remarks>
+		public override string MusicBrainzReleaseId {
+			get {return GetDashBox("com.apple.iTunes","MusicBrainz Album Id");}
+			set {SetDashBox("com.apple.iTunes", "MusicBrainz Album Id",value);}
+		}
+
+		/// <summary>
+		///    Gets and sets the MusicBrainz ReleaseArtistID
+		/// </summary>
+		/// <value>
+		///    A <see cref="string" /> containing the MusicBrainz
+		///    ReleaseArtistID for the media described by the current 
+		///    instance, or null if no value is present. 
+		/// </value>
+		/// <remarks>
+		///    This property is implemented using the "dash"/"----" box type.
+		///    http://musicbrainz.org/doc/PicardTagMapping
+		/// </remarks>
+		public override string MusicBrainzReleaseArtistId {
+			get {return GetDashBox("com.apple.iTunes","MusicBrainz Album Artist Id");}
+			set {SetDashBox("com.apple.iTunes", "MusicBrainz Album Artist Id",value);}
+		}
+
+		/// <summary>
+		///    Gets and sets the MusicBrainz TrackID
+		/// </summary>
+		/// <value>
+		///    A <see cref="string" /> containing the MusicBrainz
+		///    TrackID for the media described by the current 
+		///    instance, or null if no value is present. 
+		/// </value>
+		/// <remarks>
+		///    This property is implemented using the "dash"/"----" box type.
+		///    http://musicbrainz.org/doc/PicardTagMapping
+		/// </remarks>
+		public override string MusicBrainzTrackId {
+		    get { return GetDashBox("com.apple.iTunes","MusicIP PUID");}
+		    set {SetDashBox("com.apple.iTunes", "MusicIP PUID", value);}
+		}
+
+		/// <summary>
+		///    Gets and sets the MusicBrainz DiscID
+		/// </summary>
+		/// <value>
+		///    A <see cref="string" /> containing the MusicBrainz
+		///    DiscID for the media described by the current 
+		///    instance, or null if no value is present. 
+		/// </value>
+		/// <remarks>
+		///    This property is implemented using the "dash"/"----" box type.
+		///    http://musicbrainz.org/doc/PicardTagMapping
+		/// </remarks>
+		public override string MusicBrainzDiscId {
+			get {return GetDashBox("com.apple.iTunes","MusicBrainz Disc Id");}
+			set {SetDashBox("com.apple.iTunes", "MusicBrainz Disc Id",value);}
+		}
+
+		/// <summary>
+		///    Gets and sets the MusicIP PUID
+		/// </summary>
+		/// <value>
+		///    A <see cref="string" /> containing the MusicIP Puid
+		///    for the media described by the current 
+		///    instance, or null if no value is present. 
+		/// </value>
+		/// <remarks>
+		///    This property is implemented using the "dash"/"----" box type.
+		///    http://musicbrainz.org/doc/PicardTagMapping
+		/// </remarks>
+		public override string MusicIpId {
+			get {return GetDashBox("com.apple.iTunes","MusicIP PUID");}
+			set {SetDashBox("com.apple.iTunes", "MusicIP PUID",value);}
+		}
+
+		/// <summary>
+		///    Gets and sets the AmazonID
+		/// </summary>
+		/// <value>
+		///    A <see cref="string" /> containing the AmazonID
+		///    for the media described by the current 
+		///    instance, or null if no value is present. 
+		/// </value>
+		/// <remarks>
+		///    This property is implemented using the "dash"/"----" box type.
+		///    http://musicbrainz.org/doc/PicardTagMapping
+		/// </remarks>
+		public override string AmazonId {
+			get {return GetDashBox("com.apple.iTunes","ASIN");}
+			set {SetDashBox("com.apple.iTunes", "ASIN",value);}
+		}
+
+		/// <summary>
+		///    Gets and sets the MusicBrainz ReleaseStatus
+		/// </summary>
+		/// <value>
+		///    A <see cref="string" /> containing the MusicBrainz
+		///    ReleaseStatus for the media described by the current 
+		///    instance, or null if no value is present. 
+		/// </value>
+		/// <remarks>
+		///    This property is implemented using the "dash"/"----" box type.
+		///    http://musicbrainz.org/doc/PicardTagMapping
+		/// </remarks>
+		public override string MusicBrainzReleaseStatus {
+			get {return GetDashBox("com.apple.iTunes","MusicBrainz Album Status");}
+			set {SetDashBox("com.apple.iTunes", "MusicBrainz Album Status",value);}
+		}
+
+		/// <summary>
+		///    Gets and sets the MusicBrainz ReleaseType
+		/// </summary>
+		/// <value>
+		///    A <see cref="string" /> containing the MusicBrainz
+		///    ReleaseType for the media described by the current 
+		///    instance, or null if no value is present. 
+		/// </value>
+		/// <remarks>
+		///    This property is implemented using the "dash"/"----" box type.
+		///    http://musicbrainz.org/doc/PicardTagMapping
+		/// </remarks>
+		public override string MusicBrainzReleaseType {
+			get {return GetDashBox("com.apple.iTunes","MusicBrainz Album Type");}
+			set {SetDashBox("com.apple.iTunes", "MusicBrainz Album Type",value);}
+		}
+
+		/// <summary>
+		///    Gets and sets the MusicBrainz Release Country
+		/// </summary>
+		/// <value>
+		///    A <see cref="string" /> containing the MusicBrainz
+		///    ReleaseCountry for the media described by the current 
+		///    instance, or null if no value is present. 
+		/// </value>
+		/// <remarks>
+		///    This property is implemented using the "dash"/"----" box type.
+		///    http://musicbrainz.org/doc/PicardTagMapping
+		/// </remarks>
+		public override string MusicBrainzReleaseCountry {
+			get {return GetDashBox("com.apple.iTunes","MusicBrainz Album Release Country");}
+			set {SetDashBox("com.apple.iTunes", "MusicBrainz Album Release Country",value);}
+		}
+
 		/// <summary>
 		///    Gets and sets a collection of pictures associated with
 		///    the media represented by the current instance.
