@@ -90,6 +90,41 @@ namespace TagLib {
 		/// </summary>
 		FlacMetadata = 0x00000200,
 		
+		/// <summary>
+		///    TIFF IFD Tag
+		/// </summary>
+		TiffIFD = 0x00000400,
+
+		/// <summary>
+		///    XMP Tag
+		/// </summary>
+		XMP = 0x00000800,
+
+		/// <summary>
+		///    Jpeg Comment Tag
+		/// </summary>
+		JpegComment = 0x00001000,
+
+		/// <summary>
+		///    Gif Comment Tag
+		/// </summary>
+		GifComment = 0x00002000,
+
+		/// <summary>
+		///    native PNG keywords
+		/// </summary>
+		Png = 0x00004000,
+
+		/// <summary>
+		/// IPTC-IIM tag
+		/// </summary>
+		IPTCIIM = 0x00008000,
+
+		/// <summary>
+		///    Audible Metadata Blocks Tag
+		/// </summary>
+		AudibleMetadata = 0x00000400,
+		
 		/// <summary>
 		///    All tag types.
 		/// </summary>
